#include "js-gtk.h"

JSGTK_CLASS(Text);

static JSBool text_ctor (JSContext *cx, JSObject *obj,
			 uintN argc, jsval *argv, jsval *rval) {
    GtkWidget *text;
    GtkAdjustment *hadj, *vadj;
    if (argc < 2 || !JSVAL_IS_OBJECT(argv[0]) ||
	!JSVAL_IS_OBJECT(argv[1]))
      return JS_FALSE;
    GET_GTK(hadj,cx,JSVAL_TO_OBJECT(argv[0]),ADJUSTMENT);
    GET_GTK(vadj,cx,JSVAL_TO_OBJECT(argv[1]),ADJUSTMENT);
    text = gtk_text_new(hadj, vadj);
    JS_SetPrivate(cx,obj, text);
    jsgtkobj_registerWrapper(cx, GTK_OBJECT(text), obj);
    return JS_TRUE;
}

static JSBool text_set_editable(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkText *text;
  if (argc < 1 || !JSVAL_IS_BOOLEAN(argv[0]))
      return JS_FALSE;
  GET_GTK(text,cx,obj,TEXT);
  gtk_text_set_editable(text, JSVAL_TO_BOOLEAN(argv[0]));
  return JS_TRUE;
}


/* struct _GdkColor
{
gulong  pixel;
gushort red;
gushort green;
gushort blue;
};

struct _GdkFont
{
  GdkFontType type;
  gint ascent;
  gint descent;
};

typedef enum
{
  GDK_FONT_FONT,
  GDK_FONT_FONTSET
} GdkFontType;

*/
static JSBool text_insert_text(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkText *text;
  GdkFont gdk_font;
  GdkColor gdk_bcolor;
  GdkColor gdk_fcolor;
  JSObject *js_font;
  JSObject *js_fcolor;
  JSObject *js_bcolor;
  jsval jv;

  if (argc < 4 || !JSVAL_IS_OBJECT(argv[0]) ||
      !JSVAL_IS_OBJECT(argv[1]) ||
      !JSVAL_IS_OBJECT(argv[2]) ||
      !JSVAL_IS_STRING(argv[3]))
    return JS_FALSE;
  GET_GTK(text,cx,obj,TEXT);
  JS_ValueToObject(cx, argv[0], &js_font);
  JS_ValueToObject(cx, argv[1], &js_fcolor);
  JS_ValueToObject(cx, argv[2], &js_bcolor);
  /* font */
  gdk_font.type = JS_GetProperty(cx, js_font, "type", &jv)  ? JSVAL_TO_INT(jv) : 0;
  gdk_font.ascent =  JS_GetProperty(cx, js_font, "ascent", &jv)  ? JSVAL_TO_INT(jv) : 0;
  gdk_font.descent =  JS_GetProperty(cx, js_font, "descent", &jv)  ? JSVAL_TO_INT(jv) : 0;
  /* fcolor */
  gdk_fcolor.pixel = JS_GetProperty(cx, js_fcolor, "pixel", &jv) ? JSVAL_TO_INT(jv) : 0;
  gdk_fcolor.red =  JS_GetProperty(cx, js_fcolor, "red", &jv)  ? JSVAL_TO_INT(jv) : 0;
  gdk_fcolor.green = JS_GetProperty(cx, js_fcolor, "green", &jv)  ? JSVAL_TO_INT(jv) : 0;
  gdk_fcolor.blue =  JS_GetProperty(cx, js_fcolor, "blue", &jv)  ? JSVAL_TO_INT(jv) : 0;
  /* bcolor */
  gdk_bcolor.pixel = JS_GetProperty(cx, js_bcolor, "pixel", &jv)  ? JSVAL_TO_INT(jv) : 0;
  gdk_bcolor.red =  JS_GetProperty(cx, js_bcolor, "red", &jv)  ? JSVAL_TO_INT(jv) : 0;
  gdk_bcolor.green = JS_GetProperty(cx, js_bcolor, "green", &jv) ? JSVAL_TO_INT(jv) :0;
  gdk_bcolor.blue =  JS_GetProperty(cx, js_bcolor, "blue", &jv) ? JSVAL_TO_INT(jv) : 0;
/*  gtk_text_insert(text, &gdk_font, &gdk_fcolor, &gdk_bcolor, JS_GetStringBytes(JSVAL_TO_STRING(argv[3])),
		  strlen(JS_GetStringBytes(JSVAL_TO_STRING(argv[3])))); */
  gtk_text_insert(text, NULL, NULL, NULL, JS_GetStringBytes(JSVAL_TO_STRING(argv[3])),
		  strlen(JS_GetStringBytes(JSVAL_TO_STRING(argv[3]))));
  return JS_TRUE;
}

static JSBool text_set_word_wrap(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkText *text;
  if (argc < 1 || !JSVAL_IS_INT(argv[0]))
    return JS_FALSE;
  GET_GTK(text,cx,obj,TEXT);
  gtk_text_set_word_wrap(text, JSVAL_TO_INT(argv[0]));
  return JS_TRUE;
}

static JSBool text_set_line_wrap(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkText *text;
  if (argc < 1 || !JSVAL_IS_INT(argv[0]))
    return JS_FALSE;
  GET_GTK(text,cx,obj,TEXT);
  gtk_text_set_line_wrap(text, JSVAL_TO_INT(argv[0]));
  return JS_TRUE;
}

static JSBool text_set_point(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkText *text;
  if (argc < 1 || !JSVAL_IS_INT(argv[0]))
    return JS_FALSE;
  GET_GTK(text,cx,obj,TEXT);
  gtk_text_set_point(text, JSVAL_TO_INT(argv[0]));
  return JS_TRUE;
}

static JSBool text_get_point(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkText *text;
  GET_GTK(text,cx,obj,TEXT);
  *rval = INT_TO_JSVAL(gtk_text_get_point(text));  
  return JS_TRUE;
}

static JSBool text_get_length(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkText *text;
  guint g;
  GET_GTK(text,cx,obj,TEXT);
  g = gtk_text_get_length(text);
  *rval = INT_TO_JSVAL((int) g);  
  return JS_TRUE;
}

static JSFunctionSpec text_methods[] = {
    {"set_editable", text_set_editable, 1},
    {"insert_text", text_insert_text, 4},
    {"set_word_wrap", text_set_word_wrap, 1},
    {"set_line_wrap", text_set_line_wrap, 1},
    {"set_point", text_set_point, 1},    
    {"get_point", text_get_point, 0},
    {"get_length", text_get_length, 0},    
    {0}
};

JSGTK_INIT_FUNC(text,Text,2);

